import builtins
from typing import Any, overload, TypeAlias

img: TypeAlias = Any
bounding_box: TypeAlias = Any
quantized_images: TypeAlias = Any
templ: TypeAlias = Any
matches: TypeAlias = Any

dst: TypeAlias = Any
retval: TypeAlias = Any

class ColorGradient(Modality):
    def create(self, weak_threshold, num_features, strong_threshold) -> retval:
        """
        * \brief Constructor.
        *
        * \param weak_threshold   When quantizing, discard gradients with magnitude less than this.
        * \param num_features     How many features a template must contain.
        * \param strong_threshold Consider as candidate features only gradients whose norms are
        *                         larger than this.
        """

class DepthNormal(Modality):
    def create(self, distance_threshold, difference_threshold, num_features, extract_threshold) -> retval:
        """
        * \brief Constructor.
        *
        * \param distance_threshold   Ignore pixels beyond this distance.
        * \param difference_threshold When computing normals, ignore contributions of pixels whose
        *                             depth difference with the central pixel is above this threshold.
        * \param num_features         How many features a template must contain.
        * \param extract_threshold    Consider as candidate feature only if there are no differing
        *                             orientations within a distance of extract_threshold.
        """

class Detector:
    def addSyntheticTemplate(self, templates, class_id) -> retval:
        """
        * \brief Add a new object template computed by external means.
        """

    def addTemplate(self, sources, class_id, object_mask) -> tuple[retval, bounding_box]:
        """
        * \brief Add new object template.
        *
        * \param      sources      Source images, one for each modality.
        * \param      class_id     Object class ID.
        * \param      object_mask  Mask separating object from background.
        * \param[out] bounding_box Optionally return bounding box of the extracted features.
        *
        * \return Template ID, or -1 if failed to extract a valid template.
        """

    def classIds(self) -> retval:
        """"""

    def getModalities(self) -> retval:
        """
        * \brief Get the modalities used by this detector.
        *
        * You are not permitted to add/remove modalities, but you may dynamic_cast them to
        * tweak parameters.
        """

    def getT(self, pyramid_level) -> retval:
        """
        * \brief Get sampling step T at pyramid_level.
        """

    def getTemplates(self, class_id, template_id) -> retval:
        """
        * \brief Get the template pyramid identified by template_id.
        *
        * For example, with 2 modalities (Gradient, Normal) and two pyramid levels
        * (L0, L1), the order is (GradientL0, NormalL0, GradientL1, NormalL1).
        """

    def match(self, sources, threshold, class_ids=..., quantized_images=..., masks=...) -> tuple[matches, quantized_images]:
        """
        * \brief Detect objects by template matching.
        *
        * Matches globally at the lowest pyramid level, then refines locally stepping up the pyramid.
        *
        * \param      sources   Source images, one for each modality.
        * \param      threshold Similarity threshold, a percentage between 0 and 100.
        * \param[out] matches   Template matches, sorted by similarity score.
        * \param      class_ids If non-empty, only search for the desired object classes.
        * \param[out] quantized_images Optionally return vector<Mat> of quantized images.
        * \param      masks     The masks for consideration during matching. The masks should be CV_8UC1
        *                       where 255 represents a valid pixel.  If non-empty, the vector must be
        *                       the same size as sources.  Each element must be
        *                       empty or the same size as its corresponding source.
        """

    def numClasses(self) -> retval:
        """"""

    @overload
    def numTemplates(self) -> retval:
        """"""

    @overload
    def numTemplates(self, class_id) -> retval:
        """"""

    def pyramidLevels(self) -> retval:
        """
        * \brief Get number of pyramid levels used by this detector.
        """

    def read(self, fn) -> None:
        """"""

    def readClasses(self, class_ids, format=...) -> None:
        """"""

    def writeClasses(self, format=...) -> None:
        """"""

class Feature: ...
class Match: ...

class Modality:
    def name(self) -> retval:
        """"""

    def process(self, src, mask=...) -> retval:
        """
        * \brief Form a quantized image pyramid from a source image.
        *
        * \param[in] src  The source image. Type depends on the modality.
        * \param[in] mask Optional mask. If not empty, unmasked pixels are set to zero
        *                 in quantized image and cannot be extracted as features.
        """

    def read(self, fn) -> None:
        """"""

    @overload
    def create(self, modality_type) -> retval:
        """
        * \brief Create modality by name.
        *
        * The following modality types are supported:
        * - "ColorGradient"
        * - "DepthNormal"
        """

    @overload
    def create(self, fn) -> retval:
        """
        * \brief Load a modality from file.
        """

class QuantizedPyramid:
    def extractTemplate(self) -> tuple[retval, templ]:
        """
        * \brief Extract most discriminant features at current pyramid level to form a new template.
        *
        * \param[out] templ The new template.
        """

    def pyrDown(self) -> None:
        """
        * \brief Go to the next pyramid level.
        *
        * \todo Allow pyramid scale factor other than 2
        """

    def quantize(self, dst=...) -> dst:
        """
        * \brief Compute quantized image at current pyramid level for online detection.
        *
        * \param[out] dst The destination 8-bit image. For each pixel at most one bit is set,
        *                 representing its classification.
        """

class Template: ...

def ColorGradient_create(weak_threshold, num_features, strong_threshold) -> retval:
    """
    * \brief Constructor.
       *
       * \param weak_threshold   When quantizing, discard gradients with magnitude less than this.
       * \param num_features     How many features a template must contain.
       * \param strong_threshold Consider as candidate features only gradients whose norms are
       *                         larger than this.
    """

def DepthNormal_create(distance_threshold, difference_threshold, num_features, extract_threshold) -> retval:
    """
    * \brief Constructor.
       *
       * \param distance_threshold   Ignore pixels beyond this distance.
       * \param difference_threshold When computing normals, ignore contributions of pixels whose
       *                             depth difference with the central pixel is above this threshold.
       * \param num_features         How many features a template must contain.
       * \param extract_threshold    Consider as candidate feature only if there are no differing
       *                             orientations within a distance of extract_threshold.
    """

@overload
def Modality_create(modality_type) -> retval:
    """
    * \brief Create modality by name.
       *
       * The following modality types are supported:
       * - "ColorGradient"
       * - "DepthNormal"
    """

@overload
def Modality_create(modality_type) -> retval:
    """
    * \brief Load a modality from file.
    """

def colormap(quantized, dst=...) -> dst:
    """
    * \brief Debug function to colormap a quantized image for viewing.
    """

def drawFeatures(img, templates, tl, size=...) -> img:
    """
    * \brief Debug function to draw linemod features
    @param img
    @param templates see @ref Detector::addTemplate
    @param tl template bbox top-left offset see @ref Detector::addTemplate
    @param size marker size see @ref cv::drawMarker
    """

def getDefaultLINE() -> retval:
    """
    * \brief Factory function for detector using LINE algorithm with color gradients.

    Default parameter settings suitable for VGA images.
    """

def getDefaultLINEMOD() -> retval:
    """
    * \brief Factory function for detector using LINE-MOD algorithm with color gradients
    and depth normals.

    Default parameter settings suitable for VGA images.
    """



from typing import Any, TypeAlias

retval: TypeAlias = Any

def setParallelForBackend(backendName, propagateNumThreads=...) -> retval:
    """
    @brief Change OpenCV parallel_for backend

    @note This call is not thread-safe. Consider calling this function from the `main()` before any other OpenCV processing functions (and without any other created threads).
    """



from typing import Any, Final, overload, TypeAlias

from .. import functions as cv2

dstStep2: TypeAlias = Any
dstStep1: TypeAlias = Any
dst: TypeAlias = Any
retval: TypeAlias = Any

class GrayworldWB(WhiteBalancer):
    def getSaturationThreshold(self) -> retval:
        """
        @brief Maximum saturation for a pixel to be included in the
        gray-world assumption
        @see setSaturationThreshold
        """

    def setSaturationThreshold(self, val) -> None:
        """
        @copybrief getSaturationThreshold @see getSaturationThreshold
        """

class LearningBasedWB(WhiteBalancer):
    def extractSimpleFeatures(self, src, dst=...) -> dst:
        """
        @brief Implements the feature extraction part of the algorithm.

        In accordance with @cite Cheng2015 , computes the following features for the input image:
        1. Chromaticity of an average (R,G,B) tuple
        2. Chromaticity of the brightest (R,G,B) tuple (while ignoring saturated pixels)
        3. Chromaticity of the dominant (R,G,B) tuple (the one that has the highest value in the RGB histogram)
        4. Mode of the chromaticity palette, that is constructed by taking 300 most common colors according to
        the RGB histogram and projecting them on the chromaticity plane. Mode is the most high-density point
        of the palette, which is computed by a straightforward fixed-bandwidth kernel density estimator with
        a Epanechnikov kernel function.

        @param src Input three-channel image (BGR color space is assumed).
        @param dst An array of four (r,g) chromaticity tuples corresponding to the features listed above.
        """

    def getHistBinNum(self) -> retval:
        """
        @brief Defines the size of one dimension of a three-dimensional RGB histogram that is used internally
        by the algorithm. It often makes sense to increase the number of bins for images with higher bit depth
        (e.g. 256 bins for a 12 bit image).
        @see setHistBinNum
        """

    def getRangeMaxVal(self) -> retval:
        """
        @brief Maximum possible value of the input image (e.g. 255 for 8 bit images,
        4095 for 12 bit images)
        @see setRangeMaxVal
        """

    def getSaturationThreshold(self) -> retval:
        """
        @brief Threshold that is used to determine saturated pixels, i.e. pixels where at least one of the
        channels exceeds \f$\texttt{saturation_threshold}\times\texttt{range_max_val}\f$ are ignored.
        @see setSaturationThreshold
        """

    def setHistBinNum(self, val) -> None:
        """
        @copybrief getHistBinNum @see getHistBinNum
        """

    def setRangeMaxVal(self, val) -> None:
        """
        @copybrief getRangeMaxVal @see getRangeMaxVal
        """

    def setSaturationThreshold(self, val) -> None:
        """
        @copybrief getSaturationThreshold @see getSaturationThreshold
        """

class SimpleWB(WhiteBalancer):
    def getInputMax(self) -> retval:
        """
        @brief Input image range maximum value
        @see setInputMax
        """

    def getInputMin(self) -> retval:
        """
        @brief Input image range minimum value
        @see setInputMin
        """

    def getOutputMax(self) -> retval:
        """
        @brief Output image range maximum value
        @see setOutputMax
        """

    def getOutputMin(self) -> retval:
        """
        @brief Output image range minimum value
        @see setOutputMin
        """

    def getP(self) -> retval:
        """
        @brief Percent of top/bottom values to ignore
        @see setP
        """

    def setInputMax(self, val) -> None:
        """
        @copybrief getInputMax @see getInputMax
        """

    def setInputMin(self, val) -> None:
        """
        @copybrief getInputMin @see getInputMin
        """

    def setOutputMax(self, val) -> None:
        """
        @copybrief getOutputMax @see getOutputMax
        """

    def setOutputMin(self, val) -> None:
        """
        @copybrief getOutputMin @see getOutputMin
        """

    def setP(self, val) -> None:
        """
        @copybrief getP @see getP
        """

class TonemapDurand(cv2.Tonemap):
    def getContrast(self) -> retval:
        """"""

    def getSaturation(self) -> retval:
        """"""

    def getSigmaColor(self) -> retval:
        """"""

    def getSigmaSpace(self) -> retval:
        """"""

    def setContrast(self, contrast) -> None:
        """"""

    def setSaturation(self, saturation) -> None:
        """"""

    def setSigmaColor(self, sigma_color) -> None:
        """"""

    def setSigmaSpace(self, sigma_space) -> None:
        """"""

class WhiteBalancer(cv2.Algorithm):
    def balanceWhite(self, src, dst=...) -> dst:
        """
        @brief Applies white balancing to the input image

        @param src Input image
        @param dst White balancing result @sa cvtColor, equalizeHist
        """

@overload
def applyChannelGains(src, gainB, gainG, gainR, dst=...) -> dst:
    """
    @brief Implements an efficient fixed-point approximation for applying channel gains, which is
    """

@overload
def applyChannelGains(src, gainB, gainG, gainR, dst=...) -> dst:
    """

    @param src Input three-channel image in the BGR color space (either CV_8UC3 or CV_16UC3)
    @param dst Output image of the same size and type as src.
    @param gainB gain for the B channel
    @param gainG gain for the G channel
    @param gainR gain for the R channel
    """

@overload
def bm3dDenoising(src, dstStep1, dstStep2=..., h=..., templateWindowSize=..., searchWindowSize=..., blockMatchingStep1=..., blockMatchingStep2=..., groupSize=..., slidingStep=..., beta=..., normType=..., step=..., transformType=...) -> tuple[dstStep1, dstStep2]:
    """
    @brief Performs image denoising using the Block-Matching and 3D-filtering algorithm
            <http://www.cs.tut.fi/~foi/GCF-BM3D/BM3D_TIP_2007.pdf> with several computational
            optimizations. Noise expected to be a gaussian white noise.

            @param src Input 8-bit or 16-bit 1-channel image.
            @param dstStep1 Output image of the first step of BM3D with the same size and type as src.
            @param dstStep2 Output image of the second step of BM3D with the same size and type as src.
            @param h Parameter regulating filter strength. Big h value perfectly removes noise but also
            removes image details, smaller h value preserves details but also preserves some noise.
            @param templateWindowSize Size in pixels of the template patch that is used for block-matching.
            Should be power of 2.
            @param searchWindowSize Size in pixels of the window that is used to perform block-matching.
            Affect performance linearly: greater searchWindowsSize - greater denoising time.
            Must be larger than templateWindowSize.
            @param blockMatchingStep1 Block matching threshold for the first step of BM3D (hard thresholding),
            i.e. maximum distance for which two blocks are considered similar.
            Value expressed in euclidean distance.
            @param blockMatchingStep2 Block matching threshold for the second step of BM3D (Wiener filtering),
            i.e. maximum distance for which two blocks are considered similar.
            Value expressed in euclidean distance.
            @param groupSize Maximum size of the 3D group for collaborative filtering.
            @param slidingStep Sliding step to process every next reference block.
            @param beta Kaiser window parameter that affects the sidelobe attenuation of the transform of the
            window. Kaiser window is used in order to reduce border effects. To prevent usage of the window,
            set beta to zero.
            @param normType Norm used to calculate distance between blocks. L2 is slower than L1
            but yields more accurate results.
            @param step Step of BM3D to be executed. Possible variants are: step 1, step 2, both steps.
            @param transformType Type of the orthogonal transform used in collaborative filtering step.
            Currently only Haar transform is supported.

            This function expected to be applied to grayscale images. Advanced usage of this function
            can be manual denoising of colored image in different colorspaces.

            @sa
            fastNlMeansDenoising
    """

@overload
def bm3dDenoising(src, dstStep1, dstStep2=..., h=..., templateWindowSize=..., searchWindowSize=..., blockMatchingStep1=..., blockMatchingStep2=..., groupSize=..., slidingStep=..., beta=..., normType=..., step=..., transformType=...) -> tuple[dstStep1, dstStep2]:
    """
    @brief Performs image denoising using the Block-Matching and 3D-filtering algorithm
            <http://www.cs.tut.fi/~foi/GCF-BM3D/BM3D_TIP_2007.pdf> with several computational
            optimizations. Noise expected to be a gaussian white noise.

            @param src Input 8-bit or 16-bit 1-channel image.
            @param dst Output image with the same size and type as src.
            @param h Parameter regulating filter strength. Big h value perfectly removes noise but also
            removes image details, smaller h value preserves details but also preserves some noise.
            @param templateWindowSize Size in pixels of the template patch that is used for block-matching.
            Should be power of 2.
            @param searchWindowSize Size in pixels of the window that is used to perform block-matching.
            Affect performance linearly: greater searchWindowsSize - greater denoising time.
            Must be larger than templateWindowSize.
            @param blockMatchingStep1 Block matching threshold for the first step of BM3D (hard thresholding),
            i.e. maximum distance for which two blocks are considered similar.
            Value expressed in euclidean distance.
            @param blockMatchingStep2 Block matching threshold for the second step of BM3D (Wiener filtering),
            i.e. maximum distance for which two blocks are considered similar.
            Value expressed in euclidean distance.
            @param groupSize Maximum size of the 3D group for collaborative filtering.
            @param slidingStep Sliding step to process every next reference block.
            @param beta Kaiser window parameter that affects the sidelobe attenuation of the transform of the
            window. Kaiser window is used in order to reduce border effects. To prevent usage of the window,
            set beta to zero.
            @param normType Norm used to calculate distance between blocks. L2 is slower than L1
            but yields more accurate results.
            @param step Step of BM3D to be executed. Allowed are only BM3D_STEP1 and BM3D_STEPALL.
            BM3D_STEP2 is not allowed as it requires basic estimate to be present.
            @param transformType Type of the orthogonal transform used in collaborative filtering step.
            Currently only Haar transform is supported.

            This function expected to be applied to grayscale images. Advanced usage of this function
            can be manual denoising of colored image in different colorspaces.

            @sa
            fastNlMeansDenoising
    """

def createGrayworldWB() -> retval:
    """
    @brief Creates an instance of GrayworldWB
    """

def createLearningBasedWB(path_to_model=...) -> retval:
    """
    @brief Creates an instance of LearningBasedWB

    @param path_to_model Path to a .yml file with the model. If not specified, the default model is used
    """

def createSimpleWB() -> retval:
    """
    @brief Creates an instance of SimpleWB
    """

def createTonemapDurand(gamma=..., contrast=..., saturation=..., sigma_color=..., sigma_space=...) -> retval:
    """
    @brief Creates TonemapDurand object

    You need to set the OPENCV_ENABLE_NONFREE option in cmake to use those. Use them at your own risk.

    @param gamma gamma value for gamma correction. See createTonemap
    @param contrast resulting contrast on logarithmic scale, i. e. log(max / min), where max and min
    are maximum and minimum luminance values of the resulting image.
    @param saturation saturation enhancement value. See createTonemapDrago
    @param sigma_color bilateral filter sigma in color space
    @param sigma_space bilateral filter sigma in coordinate space
    """

def dctDenoising(src, dst, sigma, psize=...) -> None:
    """
    @brief The function implements simple dct-based denoising

        <http://www.ipol.im/pub/art/2011/ys-dct/>.
        @param src source image
        @param dst destination image
        @param sigma expected noise standard deviation
        @param psize size of block side where dct is computed

        @sa
           fastNlMeansDenoising
    """

@overload
def inpaint(src, mask, dst, algorithmType) -> None:
    """
    @brief The function implements different single-image inpainting algorithms.
    """

@overload
def inpaint(src, mask, dst, algorithmType) -> None:
    """

    @param src source image
    - #INPAINT_SHIFTMAP: it could be of any type and any number of channels from 1 to 4. In case of
    3- and 4-channels images the function expect them in CIELab colorspace or similar one, where first
    """

@overload
def inpaint(src, mask, dst, algorithmType) -> None:
    """ """

@overload
def inpaint(src, mask, dst, algorithmType) -> None:
    """
    - #INPAINT_FSR_BEST or #INPAINT_FSR_FAST: 1-channel grayscale or 3-channel BGR image.
    @param mask mask (#CV_8UC1), where non-zero pixels indicate valid image area, while zero pixels
    """

@overload
def inpaint(src, mask, dst, algorithmType) -> None:
    """
    @param dst destination image
    @param algorithmType see xphoto::InpaintTypes
    """

@overload
def oilPainting(src, size, dynRatio, code, dst=...) -> dst:
    """
    @brief oilPainting
    See the book @cite Holzmann1988 for details.
    @param src Input three-channel or one channel image (either CV_8UC3 or CV_8UC1)
    @param dst Output image of the same size and type as src.
    @param size neighbouring size is 2-size+1
    @param dynRatio image is divided by dynRatio before histogram processing
    @param code color space conversion code(see ColorConversionCodes). Histogram will used only first plane
    """

@overload
def oilPainting(src, size, dynRatio, code, dst=...) -> dst:
    """
    @brief oilPainting
    See the book @cite Holzmann1988 for details.
    @param src Input three-channel or one channel image (either CV_8UC3 or CV_8UC1)
    @param dst Output image of the same size and type as src.
    @param size neighbouring size is 2-size+1
    @param dynRatio image is divided by dynRatio before histogram processing
    """

BM3D_STEP1: Final[int]
BM3D_STEP2: Final[int]
BM3D_STEPALL: Final[int]
HAAR: Final[int]
INPAINT_FSR_BEST: Final[int]
INPAINT_FSR_FAST: Final[int]
INPAINT_SHIFTMAP: Final[int]



import builtins
from typing import Any, Final, TypeAlias

dst: TypeAlias = Any
retval: TypeAlias = Any

class ColorCorrectionModel:
    def getCCM(self) -> retval:
        """"""

    def getLoss(self) -> retval:
        """"""

    def getMask(self) -> retval:
        """"""

    def getWeights(self) -> retval:
        """"""

    def get_dst_rgbl(self) -> retval:
        """"""

    def get_src_rgbl(self) -> retval:
        """"""

    def infer(self, img, islinear=...) -> retval:
        """
        @brief Infer using fitting ccm.
        @param img the input image.
        @param islinear default false. @return the output array.
        """

    def run(self) -> None:
        """
        @brief make color correction
        """

    def setCCM_TYPE(self, ccm_type) -> None:
        """
        @brief set ccm_type
        @param ccm_type the shape of color correction matrix(CCM);\n default: @ref CCM_3x3
        """

    def setColorSpace(self, cs) -> None:
        """
        @brief set ColorSpace
        @note It should be some RGB color space;
        Supported list of color cards:
        - @ref COLOR_SPACE_sRGB
        - @ref COLOR_SPACE_AdobeRGB
        - @ref COLOR_SPACE_WideGamutRGB
        - @ref COLOR_SPACE_ProPhotoRGB
        - @ref COLOR_SPACE_DCI_P3_RGB
        - @ref COLOR_SPACE_AppleRGB
        - @ref COLOR_SPACE_REC_709_RGB
        - @ref COLOR_SPACE_REC_2020_RGB
        @param cs the absolute color space that detected colors convert to;\n default: @ref COLOR_SPACE_sRGB
        """

    def setDistance(self, distance) -> None:
        """
        @brief set Distance
        @param distance the type of color distance;\n default: @ref DISTANCE_CIE2000
        """

    def setEpsilon(self, epsilon) -> None:
        """
        @brief set Epsilon
        @param epsilon used in MinProblemSolver-DownhillSolver;\n Terminal criteria to the algorithm;\n default: 1e-4;
        """

    def setInitialMethod(self, initial_method_type) -> None:
        """
        @brief set InitialMethod
        @param initial_method_type the method of calculating CCM initial value;\n default: INITIAL_METHOD_LEAST_SQUARE
        """

    def setLinear(self, linear_type) -> None:
        """
        @brief set Linear
        @param linear_type the method of linearization;\n default: @ref LINEARIZATION_GAMMA
        """

    def setLinearDegree(self, deg) -> None:
        """
        @brief set degree
        @note only valid when linear is set to
        - @ref LINEARIZATION_COLORPOLYFIT
        - @ref LINEARIZATION_GRAYPOLYFIT
        - @ref LINEARIZATION_COLORLOGPOLYFIT
        - @ref LINEARIZATION_GRAYLOGPOLYFIT

        @param deg the degree of linearization polynomial;\n default: 3
        """

    def setLinearGamma(self, gamma) -> None:
        """
        @brief set Gamma

        @note only valid when linear is set to "gamma";\n

        @param gamma the gamma value of gamma correction;\n default: 2.2;
        """

    def setMaxCount(self, max_count) -> None:
        """
        @brief set MaxCount
        @param max_count used in MinProblemSolver-DownhillSolver;\n Terminal criteria to the algorithm;\n default: 5000;
        """

    def setSaturatedThreshold(self, lower, upper) -> None:
        """
        @brief set SaturatedThreshold.
        The colors in the closed interval [lower, upper] are reserved to participate
        in the calculation of the loss function and initialization parameters
        @param lower the lower threshold to determine saturation;\n default: 0;
        @param upper the upper threshold to determine saturation;\n default: 0
        """

    def setWeightCoeff(self, weights_coeff) -> None:
        """
        @brief set WeightCoeff
        @param weights_coeff the exponent number of L* component of the reference color in CIE Lab color space;\n default: 0
        """

    def setWeightsList(self, weights_list) -> None:
        """
        @brief set WeightsList
        @param weights_list the list of weight of each color;\n default: empty array
        """

CCM_3X3: Final[int]
CCM_3x3: Final[int]
CCM_4X3: Final[int]
CCM_4x3: Final[int]
COLORCHECKER_DIGITAL_SG: Final[int]
COLORCHECKER_DigitalSG: Final[int]
COLORCHECKER_MACBETH: Final[int]
COLORCHECKER_Macbeth: Final[int]
COLORCHECKER_VINYL: Final[int]
COLORCHECKER_Vinyl: Final[int]
COLOR_SPACE_ADOBE_RGB: Final[int]
COLOR_SPACE_ADOBE_RGBL: Final[int]
COLOR_SPACE_APPLE_RGB: Final[int]
COLOR_SPACE_APPLE_RGBL: Final[int]
COLOR_SPACE_AdobeRGB: Final[int]
COLOR_SPACE_AdobeRGBL: Final[int]
COLOR_SPACE_AppleRGB: Final[int]
COLOR_SPACE_AppleRGBL: Final[int]
COLOR_SPACE_DCI_P3_RGB: Final[int]
COLOR_SPACE_DCI_P3_RGBL: Final[int]
COLOR_SPACE_LAB_A_10: int
COLOR_SPACE_LAB_A_2: Final[int]
COLOR_SPACE_LAB_D50_10: int
COLOR_SPACE_LAB_D50_2: int
COLOR_SPACE_LAB_D55_10: int
COLOR_SPACE_LAB_D55_2: Final[int]
COLOR_SPACE_LAB_D65_10: int
COLOR_SPACE_LAB_D65_2: Final[int]
COLOR_SPACE_LAB_D75_10: int
COLOR_SPACE_LAB_D75_2: Final[int]
COLOR_SPACE_LAB_E_10: int
COLOR_SPACE_LAB_E_2: Final[int]
COLOR_SPACE_Lab_A_10: int
COLOR_SPACE_Lab_A_2: Final[int]
COLOR_SPACE_Lab_D50_10: int
COLOR_SPACE_Lab_D50_2: int
COLOR_SPACE_Lab_D55_10: int
COLOR_SPACE_Lab_D55_2: Final[int]
COLOR_SPACE_Lab_D65_10: int
COLOR_SPACE_Lab_D65_2: Final[int]
COLOR_SPACE_Lab_D75_10: int
COLOR_SPACE_Lab_D75_2: Final[int]
COLOR_SPACE_Lab_E_10: int
COLOR_SPACE_Lab_E_2: Final[int]
COLOR_SPACE_PRO_PHOTO_RGB: Final[int]
COLOR_SPACE_PRO_PHOTO_RGBL: Final[int]
COLOR_SPACE_ProPhotoRGB: Final[int]
COLOR_SPACE_ProPhotoRGBL: Final[int]
COLOR_SPACE_REC_2020_RGB: int
COLOR_SPACE_REC_2020_RGBL: int
COLOR_SPACE_REC_709_RGB: int
COLOR_SPACE_REC_709_RGBL: int
COLOR_SPACE_S_RGB: Final[int]
COLOR_SPACE_S_RGBL: Final[int]
COLOR_SPACE_WIDE_GAMUT_RGB: Final[int]
COLOR_SPACE_WIDE_GAMUT_RGBL: Final[int]
COLOR_SPACE_WideGamutRGB: Final[int]
COLOR_SPACE_WideGamutRGBL: Final[int]
COLOR_SPACE_XYZ_A_10: int
COLOR_SPACE_XYZ_A_2: Final[int]
COLOR_SPACE_XYZ_D50_10: int
COLOR_SPACE_XYZ_D50_2: int
COLOR_SPACE_XYZ_D55_10: int
COLOR_SPACE_XYZ_D55_2: Final[int]
COLOR_SPACE_XYZ_D65_10: int
COLOR_SPACE_XYZ_D65_2: Final[int]
COLOR_SPACE_XYZ_D75_10: int
COLOR_SPACE_XYZ_D75_2: Final[int]
COLOR_SPACE_XYZ_E_10: int
COLOR_SPACE_XYZ_E_2: Final[int]
COLOR_SPACE_sRGB: Final[int]
COLOR_SPACE_sRGBL: Final[int]
DISTANCE_CIE2000: int
DISTANCE_CIE76: Final[int]
DISTANCE_CIE94_GRAPHIC_ARTS: Final[int]
DISTANCE_CIE94_TEXTILES: Final[int]
DISTANCE_CMC_1TO1: Final[int]
DISTANCE_CMC_2TO1: Final[int]
DISTANCE_RGB: Final[int]
DISTANCE_RGBL: Final[int]
INITIAL_METHOD_LEAST_SQUARE: Final[int]
INITIAL_METHOD_WHITE_BALANCE: Final[int]
LINEARIZATION_COLORLOGPOLYFIT: Final[int]
LINEARIZATION_COLORPOLYFIT: Final[int]
LINEARIZATION_GAMMA: Final[int]
LINEARIZATION_GRAYLOGPOLYFIT: Final[int]
LINEARIZATION_GRAYPOLYFIT: Final[int]
LINEARIZATION_IDENTITY: Final[int]



import builtins
from typing import Any, TypeAlias

linTr: TypeAlias = Any
img2: TypeAlias = Any
shift: TypeAlias = Any
projTr: TypeAlias = Any

retval: TypeAlias = Any

class Map:
    def compose(self, map) -> None:
        """"""

    def inverseMap(self) -> retval:
        """"""

    def inverseWarp(self, img1, img2=...) -> img2:
        """"""

    def scale(self, factor) -> None:
        """"""

    def warp(self, img1, img2=...) -> img2:
        """"""

class MapAffine(Map):
    def compose(self, map) -> None:
        """"""

    def getLinTr(self, linTr=...) -> linTr:
        """"""

    def getShift(self, shift=...) -> shift:
        """"""

    def inverseMap(self) -> retval:
        """"""

    def inverseWarp(self, img1, img2=...) -> img2:
        """"""

    def scale(self, factor) -> None:
        """"""

class MapProjec(Map):
    def compose(self, map) -> None:
        """"""

    def getProjTr(self, projTr=...) -> projTr:
        """"""

    def inverseMap(self) -> retval:
        """"""

    def inverseWarp(self, img1, img2=...) -> img2:
        """"""

    def normalize(self) -> None:
        """"""

    def scale(self, factor) -> None:
        """"""

class MapShift(Map):
    def compose(self, map) -> None:
        """"""

    def getShift(self, shift=...) -> shift:
        """"""

    def inverseMap(self) -> retval:
        """"""

    def inverseWarp(self, img1, img2=...) -> img2:
        """"""

    def scale(self, factor) -> None:
        """"""

class MapTypeCaster:
    def toAffine(self, sourceMap) -> retval:
        """"""

    def toProjec(self, sourceMap) -> retval:
        """"""

    def toShift(self, sourceMap) -> retval:
        """"""

class Mapper:
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperGradAffine(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperGradEuclid(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperGradProj(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperGradShift(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperGradSimilar(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

class MapperPyramid(Mapper):
    def calculate(self, img1, img2, init=...) -> retval:
        """"""

    def getMap(self) -> retval:
        """"""

def MapTypeCaster_toAffine(sourceMap) -> retval:
    """
    .
    """

def MapTypeCaster_toProjec(sourceMap) -> retval:
    """
    .
    """

def MapTypeCaster_toShift(sourceMap) -> retval:
    """
    .
    """



import argparse
import inspect
import re
from pathlib import Path

import cv2

from .processing_utils import process_class


def main() -> None:
    parser = argparse.ArgumentParser(description="Script to generate the stubs for the opencv classes.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--output-path", "-o", type=Path, default=Path("cv2_classes.pyi"), help="Output path.")
    # parser.add_argument("--one-line-docstrings", "-one", action="store_true", help="Makes docstrings fit on one line.")
    args = parser.parse_args()

    output_path: Path = args.output_path
    # one_line_docstrings: bool = args.one_line_docstrings

    stubs: list[str] = []
    for (name, member) in inspect.getmembers(cv2):
        if inspect.isclass(member):
            process_class(name, stubs)

    # Cleanup
    stubs_str = "\n".join(stubs)
    stubs_str = re.sub(r'"""\n        \n        """', '""""""', stubs_str)
    stubs_str = re.sub(r"\n        \n", "\n\n", stubs_str)

    with output_path.open("w", encoding="utf-8") as stub_file:
        stub_file.write(stubs_str)
    print("Finished")



if __name__ == "__main__":
    main()



import builtins
from typing import Any, overload, TypeAlias

from .. import functions as cv2

confidence: TypeAlias = Any
facePoints: TypeAlias = Any
points: TypeAlias = Any
image: TypeAlias = Any
label: TypeAlias = Any
landmarks: TypeAlias = Any
faces: TypeAlias = Any
features: TypeAlias = Any

retval: TypeAlias = Any

class BIF(cv2.Algorithm):
    def compute(self, image, features=...) -> features:
        """
        Computes features sby input image.
        *  @param image Input image (CV_32FC1).
        *  @param features Feature vector (CV_32FC1).
        """

    def getNumBands(self) -> retval:
        """
        @returns The number of filter bands used for computing BIF.
        """

    def getNumRotations(self) -> retval:
        """
        @returns The number of image rotations.
        """

    def create(self, num_bands=..., num_rotations=...) -> retval:
        """
        * @param num_bands The number of filter bands (<=8) used for computing BIF.
        * @param num_rotations The number of image rotations for computing BIF. * @returns Object for computing BIF.
        """

class BasicFaceRecognizer(FaceRecognizer):
    def getEigenValues(self) -> retval:
        """"""

    def getEigenVectors(self) -> retval:
        """"""

    def getLabels(self) -> retval:
        """"""

    def getMean(self) -> retval:
        """"""

    def getNumComponents(self) -> retval:
        """
        @see setNumComponents
        """

    def getProjections(self) -> retval:
        """"""

    def getThreshold(self) -> retval:
        """
        @see setThreshold
        """

    def setNumComponents(self, val) -> None:
        """
        @copybrief getNumComponents @see getNumComponents
        """

    def setThreshold(self, val) -> None:
        """
        @copybrief getThreshold @see getThreshold
        """

class EigenFaceRecognizer(BasicFaceRecognizer):
    def create(self, num_components=..., threshold=...) -> retval:
        """
        @param num_components The number of components (read: Eigenfaces) kept for this Principal Component Analysis. As a hint: There's no rule how many components (read: Eigenfaces) should be kept for good reconstruction capabilities. It is based on your input data, so experiment with the number. Keeping 80 components should almost always be sufficient.
        @param threshold The threshold applied in the prediction.  ### Notes:  -   Training and prediction must be done on grayscale images, use cvtColor to convert between the color spaces. -   **THE EIGENFACES METHOD MAKES THE ASSUMPTION, THAT THE TRAINING AND TEST IMAGES ARE OF EQUAL SIZE.** (caps-lock, because I got so many mails asking for this). You have to make sure your input data has the correct shape, else a meaningful exception is thrown. Use resize to resize the images. -   This model does not support updating.  ### Model internal data:  -   num_components see EigenFaceRecognizer::create. -   threshold see EigenFaceRecognizer::create. -   eigenvalues The eigenvalues for this Principal Component Analysis (ordered descending). -   eigenvectors The eigenvectors for this Principal Component Analysis (ordered by their eigenvalue). -   mean The sample mean calculated from the training data. -   projections The projections of the training data. -   labels The threshold applied in the prediction. If the distance to the nearest neighbor is larger than the threshold, this method returns -1.
        """

class FaceRecognizer(cv2.Algorithm):
    def getLabelInfo(self, label) -> retval:
        """
        @brief Gets string information by label.

        If an unknown label id is provided or there is no label information associated with the specified
        label id the method returns an empty string.
        """

    def getLabelsByString(self, str) -> retval:
        """
        @brief Gets vector of labels by string.

        The function searches for the labels containing the specified sub-string in the associated string
        info.
        """

    def predict(self, src) -> tuple[label, confidence]:
        """
        @brief Predicts a label and associated confidence (e.g. distance) for a given input image.

        @param src Sample image to get a prediction from.
        @param label The predicted label for the given image.
        @param confidence Associated confidence (e.g. distance) for the predicted label.  The suffix const means that prediction does not affect the internal model state, so the method can be safely called from within different threads.  The following example shows how to get a prediction from a trained model:  @code using namespace cv; // Do your initialization here (create the cv::FaceRecognizer model) ... // ... // Read in a sample image: Mat img = imread("person1/3.jpg", IMREAD_GRAYSCALE); // And get a prediction from the cv::FaceRecognizer: int predicted = model->predict(img); @endcode  Or to get a prediction and the associated confidence (e.g. distance):  @code using namespace cv; // Do your initialization here (create the cv::FaceRecognizer model) ... // ... Mat img = imread("person1/3.jpg", IMREAD_GRAYSCALE); // Some variables for the predicted label and associated confidence (e.g. distance): int predicted_label = -1; double predicted_confidence = 0.0; // Get the prediction and associated confidence from the model model->predict(img, predicted_label, predicted_confidence); @endcode
        """

    def predict_collect(self, src, collector) -> None:
        """
        @brief - if implemented - send all result of prediction to collector that can be used for somehow custom result handling
        @param src Sample image to get a prediction from.
        @param collector User-defined collector object that accepts all results  To implement this method u just have to do same internal cycle as in predict(InputArray src, CV_OUT int &label, CV_OUT double &confidence) but not try to get "best@ result, just resend it to caller side with given collector
        """

    def predict_label(self, src) -> retval:
        """
        @overload
        """

    def read(self, filename) -> None:
        """
        @brief Loads a FaceRecognizer and its model state.

        Loads a persisted model and state from a given XML or YAML file . Every FaceRecognizer has to
        overwrite FaceRecognizer::load(FileStorage& fs) to enable loading the model state.
        FaceRecognizer::load(FileStorage& fs) in turn gets called by
        FaceRecognizer::load(const String& filename), to ease saving a model.
        """

    def setLabelInfo(self, label, strInfo) -> None:
        """
        @brief Sets string info for the specified model's label.

        The string info is replaced by the provided value if it was set before for the specified label.
        """

    def train(self, src, labels) -> None:
        """
        @brief Trains a FaceRecognizer with given data and associated labels.

        @param src The training images, that means the faces you want to learn. The data has to be given as a vector\<Mat\>.
        @param labels The labels corresponding to the images have to be given either as a vector\<int\> or a Mat of type CV_32SC1.  The following source code snippet shows you how to learn a Fisherfaces model on a given set of images. The images are read with imread and pushed into a std::vector\<Mat\>. The labels of each image are stored within a std::vector\<int\> (you could also use a Mat of type CV_32SC1). Think of the label as the subject (the person) this image belongs to, so same subjects (persons) should have the same label. For the available FaceRecognizer you don't have to pay any attention to the order of the labels, just make sure same persons have the same label:  @code // holds images and labels vector<Mat> images; vector<int> labels; // using Mat of type CV_32SC1 // Mat labels(number_of_samples, 1, CV_32SC1); // images for first person images.push_back(imread("person0/0.jpg", IMREAD_GRAYSCALE)); labels.push_back(0); images.push_back(imread("person0/1.jpg", IMREAD_GRAYSCALE)); labels.push_back(0); images.push_back(imread("person0/2.jpg", IMREAD_GRAYSCALE)); labels.push_back(0); // images for second person images.push_back(imread("person1/0.jpg", IMREAD_GRAYSCALE)); labels.push_back(1); images.push_back(imread("person1/1.jpg", IMREAD_GRAYSCALE)); labels.push_back(1); images.push_back(imread("person1/2.jpg", IMREAD_GRAYSCALE)); labels.push_back(1); @endcode  Now that you have read some images, we can create a new FaceRecognizer. In this example I'll create a Fisherfaces model and decide to keep all of the possible Fisherfaces:  @code // Create a new Fisherfaces model and retain all available Fisherfaces, // this is the most common usage of this specific FaceRecognizer: // Ptr<FaceRecognizer> model =  FisherFaceRecognizer::create(); @endcode  And finally train it on the given dataset (the face images and labels):  @code // This is the common interface to train all of the available cv::FaceRecognizer // implementations: // model->train(images, labels); @endcode
        """

    def update(self, src, labels) -> None:
        """
        @brief Updates a FaceRecognizer with given data and associated labels.

        @param src The training images, that means the faces you want to learn. The data has to be given as a vector\<Mat\>.
        @param labels The labels corresponding to the images have to be given either as a vector\<int\> or a Mat of type CV_32SC1.  This method updates a (probably trained) FaceRecognizer, but only if the algorithm supports it. The Local Binary Patterns Histograms (LBPH) recognizer (see createLBPHFaceRecognizer) can be updated. For the Eigenfaces and Fisherfaces method, this is algorithmically not possible and you have to re-estimate the model with FaceRecognizer::train. In any case, a call to train empties the existing model and learns a new model, while update does not delete any model data.  @code // Create a new LBPH model (it can be updated) and use the default parameters, // this is the most common usage of this specific FaceRecognizer: // Ptr<FaceRecognizer> model =  LBPHFaceRecognizer::create(); // This is the common interface to train all of the available cv::FaceRecognizer // implementations: // model->train(images, labels); // Some containers to hold new image: vector<Mat> newImages; vector<int> newLabels; // You should add some images to the containers: // // ... // // Now updating the model is as easy as calling: model->update(newImages,newLabels); // This will preserve the old model data and extend the existing model // with the new features extracted from newImages! @endcode  Calling update on an Eigenfaces model (see EigenFaceRecognizer::create), which doesn't support updating, will throw an error similar to:  @code OpenCV Error: The function/feature is not implemented (This FaceRecognizer (FaceRecognizer.Eigenfaces) does not support updating, you have to use FaceRecognizer::train to update it.) in update, file /home/philipp/git/opencv/modules/contrib/src/facerec.cpp, line 305 terminate called after throwing an instance of 'cv::Exception' @endcode  @note The FaceRecognizer does not store your training images, because this would be very memory intense and it's not the responsibility of te FaceRecognizer to do so. The caller is responsible for maintaining the dataset, he want to work with.
        """

    def write(self, filename) -> None:
        """
        @brief Saves a FaceRecognizer and its model state.

        Saves this model to a given filename, either as XML or YAML.
        @param filename The filename to store this FaceRecognizer to (either XML/YAML).  Every FaceRecognizer overwrites FaceRecognizer::save(FileStorage& fs) to save the internal model state. FaceRecognizer::save(const String& filename) saves the state of a model to the given filename.  The suffix const means that prediction does not affect the internal model state, so the method can be safely called from within different threads.
        """

class Facemark(cv2.Algorithm):
    def fit(self, image, faces, landmarks=...) -> tuple[retval, landmarks]:
        """
        @brief Detect facial landmarks from an image.
        @param image Input image.
        @param faces Output of the function which represent region of interest of the detected faces. Each face is stored in cv::Rect container.
        @param landmarks The detected landmark points for each faces.  <B>Example of usage</B> @code Mat image = imread("image.jpg"); std::vector<Rect> faces; std::vector<std::vector<Point2f> > landmarks; facemark->fit(image, faces, landmarks); @endcode
        """

    def loadModel(self, model) -> None:
        """
        @brief A function to load the trained model before the fitting process.
        @param model A string represent the filename of a trained model.  <B>Example of usage</B> @code facemark->loadModel("../data/lbf.model"); @endcode
        """

class FacemarkAAM(FacemarkTrain): ...
class FacemarkKazemi(Facemark): ...
class FacemarkLBF(FacemarkTrain): ...
class FacemarkTrain(Facemark): ...

class FisherFaceRecognizer(BasicFaceRecognizer):
    def create(self, num_components=..., threshold=...) -> retval:
        """
        @param num_components The number of components (read: Fisherfaces) kept for this Linear Discriminant Analysis with the Fisherfaces criterion. It's useful to keep all components, that means the number of your classes c (read: subjects, persons you want to recognize). If you leave this at the default (0) or set it to a value less-equal 0 or greater (c-1), it will be set to the correct number (c-1) automatically.
        @param threshold The threshold applied in the prediction. If the distance to the nearest neighbor is larger than the threshold, this method returns -1.  ### Notes:  -   Training and prediction must be done on grayscale images, use cvtColor to convert between the color spaces. -   **THE FISHERFACES METHOD MAKES THE ASSUMPTION, THAT THE TRAINING AND TEST IMAGES ARE OF EQUAL SIZE.** (caps-lock, because I got so many mails asking for this). You have to make sure your input data has the correct shape, else a meaningful exception is thrown. Use resize to resize the images. -   This model does not support updating.  ### Model internal data:  -   num_components see FisherFaceRecognizer::create. -   threshold see FisherFaceRecognizer::create. -   eigenvalues The eigenvalues for this Linear Discriminant Analysis (ordered descending). -   eigenvectors The eigenvectors for this Linear Discriminant Analysis (ordered by their eigenvalue). -   mean The sample mean calculated from the training data. -   projections The projections of the training data. -   labels The labels corresponding to the projections.
        """

class LBPHFaceRecognizer(FaceRecognizer):
    def getGridX(self) -> retval:
        """
        @see setGridX
        """

    def getGridY(self) -> retval:
        """
        @see setGridY
        """

    def getHistograms(self) -> retval:
        """"""

    def getLabels(self) -> retval:
        """"""

    def getNeighbors(self) -> retval:
        """
        @see setNeighbors
        """

    def getRadius(self) -> retval:
        """
        @see setRadius
        """

    def getThreshold(self) -> retval:
        """
        @see setThreshold
        """

    def setGridX(self, val) -> None:
        """
        @copybrief getGridX @see getGridX
        """

    def setGridY(self, val) -> None:
        """
        @copybrief getGridY @see getGridY
        """

    def setNeighbors(self, val) -> None:
        """
        @copybrief getNeighbors @see getNeighbors
        """

    def setRadius(self, val) -> None:
        """
        @copybrief getRadius @see getRadius
        """

    def setThreshold(self, val) -> None:
        """
        @copybrief getThreshold @see getThreshold
        """

    def create(self, radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
        """
        @param radius The radius used for building the Circular Local Binary Pattern. The greater the radius, the smoother the image but more spatial information you can get.
        @param neighbors The number of sample points to build a Circular Local Binary Pattern from. An appropriate value is to use `8` sample points. Keep in mind: the more sample points you include, the higher the computational cost.
        @param grid_x The number of cells in the horizontal direction, 8 is a common value used in publications. The more cells, the finer the grid, the higher the dimensionality of the resulting feature vector.
        @param grid_y The number of cells in the vertical direction, 8 is a common value used in publications. The more cells, the finer the grid, the higher the dimensionality of the resulting feature vector.
        @param threshold The threshold applied in the prediction. If the distance to the nearest neighbor is larger than the threshold, this method returns -1.  ### Notes:  -   The Circular Local Binary Patterns (used in training and prediction) expect the data given as grayscale images, use cvtColor to convert between the color spaces. -   This model supports updating.  ### Model internal data:  -   radius see LBPHFaceRecognizer::create. -   neighbors see LBPHFaceRecognizer::create. -   grid_x see LLBPHFaceRecognizer::create. -   grid_y see LBPHFaceRecognizer::create. -   threshold see LBPHFaceRecognizer::create. -   histograms Local Binary Patterns Histograms calculated from the given training data (empty if none was given). -   labels Labels corresponding to the calculated Local Binary Patterns Histograms.
        """

class MACE(cv2.Algorithm):
    def salt(self, passphrase) -> None:
        """
        @brief optionally encrypt images with random convolution
        @param passphrase a crc64 random seed will get generated from this
        """

    def same(self, query) -> retval:
        """
        @brief correlate query img and threshold to min class value
        @param query  a Mat with query image
        """

    def train(self, images) -> None:
        """
        @brief train it on positive features
        compute the mace filter: `h = D(-1) * X * (X(+) * D(-1) * X)(-1) * C`
        also calculate a minimal threshold for this class, the smallest self-similarity from the train images
        @param images  a vector<Mat> with the train images
        """

    def create(self, IMGSIZE=...) -> retval:
        """
        @brief constructor
        @param IMGSIZE  images will get resized to this (should be an even number)
        """

    def load(self, filename, objname=...) -> retval:
        """
        @brief constructor
        @param filename  build a new MACE instance from a pre-serialized FileStorage
        @param objname (optional) top-level node in the FileStorage
        """

class PredictCollector: ...

class StandardCollector(PredictCollector):
    def getMinDist(self) -> retval:
        """
        @brief Returns minimal distance value
        """

    def getMinLabel(self) -> retval:
        """
        @brief Returns label with minimal distance
        """

    def getResults(self, sorted=...) -> retval:
        """
        @brief Return results as vector
        @param sorted If set, results will be sorted by distance Each values is a pair of label and distance.
        """

    def create(self, threshold=...) -> retval:
        """
        @brief Static constructor
        @param threshold set threshold
        """

def BIF_create(num_bands=..., num_rotations=...) -> retval:
    """
    * @param num_bands The number of filter bands (<=8) used for computing BIF.
         * @param num_rotations The number of image rotations for computing BIF.
         * @returns Object for computing BIF.
    """

@overload
def EigenFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """
    @param num_components The number of components (read: Eigenfaces) kept for this Principal
    """

@overload
def EigenFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """ """

@overload
def EigenFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """ """

@overload
def EigenFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """
    @param threshold The threshold applied in the prediction.

    ### Notes:

    -   Training and prediction must be done on grayscale images, use cvtColor to convert between the
        color spaces.
    -   **THE EIGENFACES METHOD MAKES THE ASSUMPTION, THAT THE TRAINING AND TEST IMAGES ARE OF EQUAL
        SIZE.** (caps-lock, because I got so many mails asking for this). You have to make sure your
        input data has the correct shape, else a meaningful exception is thrown. Use resize to resize
        the images.
    -   This model does not support updating.

    ### Model internal data:

    -   num_components see EigenFaceRecognizer::create.
    -   threshold see EigenFaceRecognizer::create.
    -   eigenvalues The eigenvalues for this Principal Component Analysis (ordered descending).
    -   eigenvectors The eigenvectors for this Principal Component Analysis (ordered by their
        eigenvalue).
    -   mean The sample mean calculated from the training data.
    -   projections The projections of the training data.
    -   labels The threshold applied in the prediction. If the distance to the nearest neighbor is
        larger than the threshold, this method returns -1.
    """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """
    @param num_components The number of components (read: Fisherfaces) kept for this Linear
    """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """ """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """ """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """ """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """
    @param threshold The threshold applied in the prediction. If the distance to the nearest neighbor
    """

@overload
def FisherFaceRecognizer_create(num_components=..., threshold=...) -> retval:
    """

    ### Notes:

    -   Training and prediction must be done on grayscale images, use cvtColor to convert between the
        color spaces.
    -   **THE FISHERFACES METHOD MAKES THE ASSUMPTION, THAT THE TRAINING AND TEST IMAGES ARE OF EQUAL
        SIZE.** (caps-lock, because I got so many mails asking for this). You have to make sure your
        input data has the correct shape, else a meaningful exception is thrown. Use resize to resize
        the images.
    -   This model does not support updating.

    ### Model internal data:

    -   num_components see FisherFaceRecognizer::create.
    -   threshold see FisherFaceRecognizer::create.
    -   eigenvalues The eigenvalues for this Linear Discriminant Analysis (ordered descending).
    -   eigenvectors The eigenvectors for this Linear Discriminant Analysis (ordered by their
        eigenvalue).
    -   mean The sample mean calculated from the training data.
    -   projections The projections of the training data.
    -   labels The labels corresponding to the projections.
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """
    @param radius The radius used for building the Circular Local Binary Pattern. The greater the
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """
    @param neighbors The number of sample points to build a Circular Local Binary Pattern from. An
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """ """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """
    @param grid_x The number of cells in the horizontal direction, 8 is a common value used in
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """ """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """
    @param grid_y The number of cells in the vertical direction, 8 is a common value used in
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """ """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """
    @param threshold The threshold applied in the prediction. If the distance to the nearest neighbor
    """

@overload
def LBPHFaceRecognizer_create(radius=..., neighbors=..., grid_x=..., grid_y=..., threshold=...) -> retval:
    """

    ### Notes:

    -   The Circular Local Binary Patterns (used in training and prediction) expect the data given as
        grayscale images, use cvtColor to convert between the color spaces.
    -   This model supports updating.

    ### Model internal data:

    -   radius see LBPHFaceRecognizer::create.
    -   neighbors see LBPHFaceRecognizer::create.
    -   grid_x see LLBPHFaceRecognizer::create.
    -   grid_y see LBPHFaceRecognizer::create.
    -   threshold see LBPHFaceRecognizer::create.
    -   histograms Local Binary Patterns Histograms calculated from the given training data (empty if
        none was given).
    -   labels Labels corresponding to the calculated Local Binary Patterns Histograms.
    """

def MACE_create(IMGSIZE=...) -> retval:
    """
    @brief constructor
        @param IMGSIZE  images will get resized to this (should be an even number)
    """

def MACE_load(filename, objname=...) -> retval:
    """
    @brief constructor
        @param filename  build a new MACE instance from a pre-serialized FileStorage
        @param objname (optional) top-level node in the FileStorage
    """

def StandardCollector_create(threshold=...) -> retval:
    """
    @brief Static constructor
        @param threshold set threshold
    """

def createFacemarkAAM() -> retval:
    """
    .
    """

def createFacemarkKazemi() -> retval:
    """
    .
    """

def createFacemarkLBF() -> retval:
    """
    .
    """

@overload
def drawFacemarks(image, points, color=...) -> image:
    """
    @brief Utility to draw the detected facial landmark points

    @param image The input image to be processed.
    @param points Contains the data of points which will be drawn.
    @param color The color of points in BGR format represented by cv::Scalar.

    <B>Example of usage</B>
    @code
    std::vector<Rect> faces;
    std::vector<std::vector<Point2f> > landmarks;
    facemark->getFaces(img, faces);
    facemark->fit(img, faces, landmarks);
    for(int j=0;j<rects.size();j++){
    """

@overload
def drawFacemarks(image, points, color=...) -> image:
    """
    }
    @endcode
    """

@overload
def getFacesHAAR(image, face_cascade_name, faces=...) -> tuple[retval, faces]:
    """
    @brief Default face detector
    This function is mainly utilized by the implementation of a Facemark Algorithm.
    End users are advised to use function Facemark::getFaces which can be manually defined
    and circumvented to the algorithm by Facemark::setFaceDetector.

    @param image The input image to be processed.
    @param faces Output of the function which represent region of interest of the detected faces.
    Each face is stored in cv::Rect container.
    @param params detector parameters

    <B>Example of usage</B>
    @code
    std::vector<cv::Rect> faces;
    CParams params("haarcascade_frontalface_alt.xml");
    cv::face::getFaces(frame, faces, &params);
    for(int j=0;j<faces.size();j++){
    """

@overload
def getFacesHAAR(image, face_cascade_name, faces=...) -> tuple[retval, faces]:
    """
    }
    cv::imshow("detection", frame);
    @endcode
    """

def loadDatasetList(imageList, annotationList, images, annotations) -> retval:
    """
    @brief A utility to load list of paths to training image and annotation file.
    @param imageList The specified file contains paths to the training images.
    @param annotationList The specified file contains paths to the training annotations.
    @param images The loaded paths of training images.
    @param annotations The loaded paths of annotation files.

    Example of usage:
    @code
    String imageFiles = "images_path.txt";
    String ptsFiles = "annotations_path.txt";
    std::vector<String> images_train;
    std::vector<String> landmarks_train;
    loadDatasetList(imageFiles,ptsFiles,images_train,landmarks_train);
    @endcode
    """

def loadFacePoints(filename, points=..., offset=...) -> tuple[retval, points]:
    """
    @brief A utility to load facial landmark information from a given file.

    @param filename The filename of file contains the facial landmarks data.
    @param points The loaded facial landmark points.
    @param offset An offset value to adjust the loaded points.

    <B>Example of usage</B>
    @code
    std::vector<Point2f> points;
    face::loadFacePoints("filename.txt", points, 0.0f);
    @endcode

    The annotation file should follow the default format which is
    @code
    version: 1
    n_points:  68
    {
    212.716603 499.771793
    230.232816 566.290071
    ...
    }
    @endcode
    where n_points is the number of points considered
    and each point is represented as its position in x and y.
    """

@overload
def loadTrainingData(filename, images, facePoints=..., delim=..., offset=...) -> tuple[retval, facePoints]:
    """
    @brief A utility to load facial landmark dataset from a single file.

    @param filename The filename of a file that contains the dataset information.
    Each line contains the filename of an image followed by
    pairs of x and y values of facial landmarks points separated by a space.
    Example
    @code
    /home/user/ibug/image_003_1.jpg 336.820955 240.864510 334.238298 260.922709 335.266918 ...
    /home/user/ibug/image_005_1.jpg 376.158428 230.845712 376.736984 254.924635 383.265403 ...
    @endcode
    @param images A vector where each element represent the filename of image in the dataset.
    Images are not loaded by default to save the memory.
    @param facePoints The loaded landmark points for all training data.
    @param delim Delimiter between each element, the default value is a whitespace.
    @param offset An offset value to adjust the loaded points.

    <B>Example of usage</B>
    @code
    cv::String imageFiles = "../data/images_train.txt";
    cv::String ptsFiles = "../data/points_train.txt";
    std::vector<String> images;
    std::vector<std::vector<Point2f> > facePoints;
    loadTrainingData(imageFiles, ptsFiles, images, facePoints, 0.0f);
    @endcode
    """

@overload
def loadTrainingData(filename, images, facePoints=..., delim=..., offset=...) -> tuple[retval, facePoints]:
    """
    @brief A utility to load facial landmark information from the dataset.

    @param imageList A file contains the list of image filenames in the training dataset.
    @param groundTruth A file contains the list of filenames
    where the landmarks points information are stored.
    The content in each file should follow the standard format (see face::loadFacePoints).
    @param images A vector where each element represent the filename of image in the dataset.
    Images are not loaded by default to save the memory.
    @param facePoints The loaded landmark points for all training data.
    @param offset An offset value to adjust the loaded points.

    <B>Example of usage</B>
    @code
    cv::String imageFiles = "../data/images_train.txt";
    cv::String ptsFiles = "../data/points_train.txt";
    std::vector<String> images;
    std::vector<std::vector<Point2f> > facePoints;
    loadTrainingData(imageFiles, ptsFiles, images, facePoints, 0.0f);
    @endcode

    example of content in the images_train.txt
    @code
    /home/user/ibug/image_003_1.jpg
    /home/user/ibug/image_004_1.jpg
    /home/user/ibug/image_005_1.jpg
    /home/user/ibug/image_006.jpg
    @endcode

    example of content in the points_train.txt
    @code
    /home/user/ibug/image_003_1.pts
    /home/user/ibug/image_004_1.pts
    /home/user/ibug/image_005_1.pts
    /home/user/ibug/image_006.pts
    @endcode
    """

@overload
def loadTrainingData(filename, images, facePoints=..., delim=..., offset=...) -> tuple[retval, facePoints]:
    """
    @brief This function extracts the data for training from .txt files which contains the corresponding image name and landmarks.
    *The first file in each file should give the path of the image whose
    *landmarks are being described in the file. Then in the subsequent
    *lines there should be coordinates of the landmarks in the image
    *i.e each line should be of the form x,y
    *where x represents the x coordinate of the landmark and y represents
    *the y coordinate of the landmark.
    *
    *For reference you can see the files as provided in the
    *<a href="http://www.ifp.illinois.edu/~vuongle2/helen/">HELEN dataset</a>
    *
    * @param filename A vector of type cv::String containing name of the .txt files.
    * @param trainlandmarks A vector of type cv::Point2f that would store shape or landmarks of all images.
    * @param trainimages A vector of type cv::String which stores the name of images whose landmarks are tracked
    * @returns A boolean value. It returns true when it reads the data successfully and false otherwise
    """



import builtins
from typing import Any, Final, overload, TypeAlias

from . import detail

normals: TypeAlias = Any
points: TypeAlias = Any
image: TypeAlias = Any
retval: TypeAlias = Any

class KinFu:
    def getCloud(self, points=..., normals=...) -> tuple[points, normals]:
        """
        @brief Gets points and normals of current 3d mesh

        The order of normals corresponds to order of points.
        The order of points is undefined.

        @param points vector of points which are 4-float vectors
        @param normals vector of normals which are 4-float vectors
        """

    def getNormals(self, points, normals=...) -> normals:
        """
        @brief Calculates normals for given points
        @param points input vector of points which are 4-float vectors
        @param normals output vector of corresponding normals which are 4-float vectors
        """

    def getPoints(self, points=...) -> points:
        """
        @brief Gets points of current 3d mesh

        The order of points is undefined.

        @param points vector of points which are 4-float vectors
        """

    @overload
    def render(self, image=...) -> image:
        """
        @brief Renders a volume into an image

        Renders a 0-surface of TSDF using Phong shading into a CV_8UC4 Mat.
        Light pose is fixed in KinFu params.

        @param image resulting image
        """

    @overload
    def render(self, cameraPose, image=...) -> image:
        """
        @brief Renders a volume into an image

        Renders a 0-surface of TSDF using Phong shading into a CV_8UC4 Mat.
        Light pose is fixed in KinFu params.

        @param image resulting image
        @param cameraPose pose of camera to render from. If empty then render from current pose which is a last frame camera pose.
        """

    def reset(self) -> None:
        """
        @brief Resets the algorithm

        Clears current model and resets a pose.
        """

    def update(self, depth) -> retval:
        """
        @brief Process next depth frame

        Integrates depth into voxel space with respect to its ICP-calculated pose.
        Input image is converted to CV_32F internally if has another type.

        @param depth one-channel image which size and depth scale is described in algorithm's parameters @return true if succeeded to align new frame with current scene, false if opposite
        """

    def create(self, _params) -> retval:
        """"""

class Params:
    @overload
    def setInitialVolumePose(self, R, t) -> None:
        """
        * @brief Set Initial Volume Pose
        * Sets the initial pose of the TSDF volume.
        * @param R rotation matrix
        * @param t translation vector
        """

    @overload
    def setInitialVolumePose(self, homogen_tf) -> None:
        """
        * @brief Set Initial Volume Pose
        * Sets the initial pose of the TSDF volume.
        * @param homogen_tf 4 by 4 Homogeneous Transform matrix to set the intial pose of TSDF volume
        """

    def coarseParams(self) -> retval:
        """
        @brief Coarse parameters
        A set of parameters which provides better speed, can fail to match frames
        in case of rapid sensor motion.
        """

    def coloredTSDFParams(self, isCoarse) -> retval:
        """
        @brief ColoredTSDF parameters
        A set of parameters suitable for use with ColoredTSDFVolume
        """

    def defaultParams(self) -> retval:
        """
        * @brief Default parameters
        * A set of parameters which provides better model quality, can be very slow.
        """

    def hashTSDFParams(self, isCoarse) -> retval:
        """
        @brief HashTSDF parameters
        A set of parameters suitable for use with HashTSDFVolume
        """

class Volume: ...

class VolumeParams:
    def coarseParams(self, _volumeType) -> retval:
        """
        @brief Coarse set of parameters that provides relatively higher performance
        at the cost of reconstrution quality.
        """

    def defaultParams(self, _volumeType) -> retval:
        """
        @brief Default set of parameters that provide higher quality reconstruction
        at the cost of slow performance.
        """

def KinFu_create(_params) -> retval:
    """
    .
    """

@overload
def Params_coarseParams() -> retval:
    """
    @brief Coarse parameters
    """

@overload
def Params_coarseParams() -> retval:
    """ """

@overload
def Params_coarseParams() -> retval:
    """ """

def Params_coloredTSDFParams(isCoarse) -> retval:
    """
    @brief ColoredTSDF parameters
          A set of parameters suitable for use with ColoredTSDFVolume
    """

def Params_defaultParams() -> retval:
    """
    * @brief Default parameters
         * A set of parameters which provides better model quality, can be very slow.
    """

def Params_hashTSDFParams(isCoarse) -> retval:
    """
    @brief HashTSDF parameters
          A set of parameters suitable for use with HashTSDFVolume
    """

def VolumeParams_coarseParams(_volumeType) -> retval:
    """
    @brief Coarse set of parameters that provides relatively higher performance
            at the cost of reconstrution quality.
    """

def VolumeParams_defaultParams(_volumeType) -> retval:
    """
    @brief Default set of parameters that provide higher quality reconstruction
            at the cost of slow performance.
    """

def makeVolume(_volumeType, _voxelSize, _pose, _raycastStepFactor, _truncDist, _maxWeight, _truncateThreshold, _resolution) -> retval:
    """
    .
    """

VOLUME_TYPE_COLOREDTSDF: Final[int]
VOLUME_TYPE_HASHTSDF: Final[int]
VOLUME_TYPE_TSDF: Final[int]
VolumeType_COLOREDTSDF: Final[int]
VolumeType_HASHTSDF: Final[int]
VolumeType_TSDF: Final[int]



import builtins
from typing import Any, TypeAlias

contour: TypeAlias = Any

retval: TypeAlias = Any

class IntelligentScissorsMB:
    def applyImage(self, image) -> retval:
        """
        @brief Specify input image and extract image features
        *
        * @param image input image. Type is #CV_8UC1 / #CV_8UC3
        """

    def applyImageFeatures(self, non_edge, gradient_direction, gradient_magnitude, image=...) -> retval:
        """
        @brief Specify custom features of input image
        *
        * Customized advanced variant of applyImage() call.
        *
        * @param non_edge Specify cost of non-edge pixels. Type is CV_8UC1. Expected values are `{0, 1}`.
        * @param gradient_direction Specify gradient direction feature. Type is CV_32FC2. Values are expected to be normalized: `x^2 + y^2 == 1`
        * @param gradient_magnitude Specify cost of gradient magnitude function: Type is CV_32FC1. Values should be in range `[0, 1]`.
        * @param image **Optional parameter**. Must be specified if subset of features is specified (non-specified features are calculated internally)
        """

    def buildMap(self, sourcePt) -> None:
        """
        @brief Prepares a map of optimal paths for the given source point on the image
        *
        * @note applyImage() / applyImageFeatures() must be called before this call
        *
        * @param sourcePt The source point used to find the paths
        """

    def getContour(self, targetPt, contour=..., backward=...) -> contour:
        """
        @brief Extracts optimal contour for the given target point on the image
        *
        * @note buildMap() must be called before this call
        *
        * @param targetPt The target point
        * @param[out] contour The list of pixels which contains optimal path between the source and the target points of the image. Type is CV_32SC2 (compatible with `std::vector<Point>`)
        * @param backward Flag to indicate reverse order of retrived pixels (use "true" value to fetch points from the target to the source point)
        """

    def setEdgeFeatureCannyParameters(self, threshold1, threshold2, apertureSize=..., L2gradient=...) -> retval:
        """
        @brief Switch edge feature extractor to use Canny edge detector
        *
        * @note "Laplacian Zero-Crossing" feature extractor is used by default (following to original article)
        *
        * @sa Canny
        """

    def setEdgeFeatureZeroCrossingParameters(self, gradient_magnitude_min_value=...) -> retval:
        """
        @brief Switch to "Laplacian Zero-Crossing" edge feature extractor and specify its parameters
        *
        * This feature extractor is used by default according to article.
        *
        * Implementation has additional filtering for regions with low-amplitude noise.
        * This filtering is enabled through parameter of minimal gradient amplitude (use some small value 4, 8, 16).
        *
        * @note Current implementation of this feature extractor is based on processing of grayscale images (color image is converted to grayscale image first).
        *
        * @note Canny edge detector is a bit slower, but provides better results (especially on color images): use setEdgeFeatureCannyParameters().
        *
        * @param gradient_magnitude_min_value Minimal gradient magnitude value for edge pixels (default: 0, check is disabled)
        """

    def setGradientMagnitudeMaxLimit(self, gradient_magnitude_threshold_max=...) -> retval:
        """
        @brief Specify gradient magnitude max value threshold
        *
        * Zero limit value is used to disable gradient magnitude thresholding (default behavior, as described in original article).
        * Otherwize pixels with `gradient magnitude >= threshold` have zero cost.
        *
        * @note Thresholding should be used for images with irregular regions (to avoid stuck on parameters from high-contract areas, like embedded logos).
        *
        * @param gradient_magnitude_threshold_max Specify gradient magnitude max value threshold (default: 0, disabled)
        """

    def setWeights(self, weight_non_edge, weight_gradient_direction, weight_gradient_magnitude) -> retval:
        """
        @brief Specify weights of feature functions
        *
        * Consider keeping weights normalized (sum of weights equals to 1.0)
        * Discrete dynamic programming (DP) goal is minimization of costs between pixels.
        *
        * @param weight_non_edge Specify cost of non-edge pixels (default: 0.43f)
        * @param weight_gradient_direction Specify cost of gradient direction function (default: 0.43f)
        * @param weight_gradient_magnitude Specify cost of gradient magnitude function (default: 0.14f)
        """
